﻿using System;
using System.Net.Mail;

using IsolationFrameworkDemo.Entity;

namespace IsolationFrameworkDemo.Services
{
    public interface IUserLoginService 
    {
        void EmailLoginInformation( ChallengeResponse challengeResponse, string userName );
    }

    public class UserLoginService : IUserLoginService
    {
        public UserLoginService( IUserRepository userRepository, 
            IChallangeValidator challangeValidator, 
            IEmailGateway emailGateway )
        {
            UserRepository = userRepository;
            ChallangeValidator = challangeValidator;
            EmailGateway = emailGateway;
        }

        public void EmailLoginInformation( ChallengeResponse challengeResponse, string userName )
        {
            User validUserInformation = UserRepository.FetchUserInformation( userName );
            
            if ( ( validUserInformation == null ) ) { throw new Exception( "The Username provided was not valid." ); }
            
            ChallengeResponse databaseChallangeResponse = UserRepository.GetChallangeResponseInformation( validUserInformation );

            if ( !ChallangeValidator.ValidateUserInformation( databaseChallangeResponse, 
                challengeResponse ) )
            {
                throw new Exception( "The Challenge/Response Information provided was not valid." );
            }

            try
            {
            
                EmailGateway.SendEmail();                
            }
            catch ( SmtpException )
            {
                throw new Exception( "There was an issue sending your email, we will try again shortly." );
            }

        }

        private IUserRepository UserRepository { get; set; }
        private IChallangeValidator ChallangeValidator { get; set; }
        private IEmailGateway EmailGateway { get; set; }
    }
}
